using System;
using Lazy;

// Parcel of Land = Lot

// LAZY METHODS must be declared 'virtual'

namespace Lazy.Example
{
	// Interface

	public interface Lot {
		float GetArea();
		int   GetTrees();

		float Area { get; set; }
		Image Photograph { get; set; }
		Rectangle MBR { get; set; }
		Polygon Perimeter { get; set; }
	}	

	// Implementation (with alternatives)

	[Lazy]
	public class LotImplementation: Lot {

		private float     area;       // Actual area
		private Rectangle mbr;        // Lot MBR (minimum bounding rectangle)
		private Polygon   perimeter;  // Lot limits
		private Image     photograph; // Aerial image

		// Properties

		public float Area {
			get { return GetArea(); }
			set { area = value; }
		}

		public Image Photograph {
			get { return photograph; }
			set { photograph = value; }
		}

		public Rectangle MBR {
			get { return mbr; }
			set { mbr = value; }
		}

		public Polygon Perimeter {
			get { return perimeter; }
			set { perimeter = value; }
		}


		// Area

		public virtual float GetArea () {
			return area;
		}

		[AlternativeImplementation("GetArea")]
		public float GetAreaFromMBR () {

			if (mbr!=null)
                return mbr.GetArea();
			else
				return 0;
		}

		[AlternativeImplementation("GetArea")]
		public float GetAreaFromPerimeter () {

			if (perimeter!=null)
				return perimeter.GetArea();
			else
				return 0;
		}


		// Trees
		
		private const float AverageTreeDensity = 0.2f;
		private const float AverageTreeSize = 3;

		public virtual int GetTrees () {
			return (int) ( GetArea()* AverageTreeDensity );
		}

		[AlternativeImplementation("GetTrees")]
		public int GetTreesFromPhoto () {
			return ImageMorphologyAnalyzer.GetObjectCount (photograph, perimeter, AverageTreeSize);
		}

		// Extra

		public virtual int GetTrees (int n) {
			return GetTrees()/n;
		}

		[AlternativeImplementation("GetTrees")]
		public int GetTreesFromPhoto (int n) {
			return GetTreesFromPhoto()/n;
		}
	}


	public class LotSubclass: LotImplementation {

		public override float GetArea () {

			return 2*base.GetArea();
		}
	}





	





}
