using System;

using Lazy;

using NUnit.Framework;


namespace Lazy.Example
{
	/// <summary>
	/// Descripcin breve de TriangleTest.
	/// </summary>
	[TestFixture]
	public class TriangleTest
	{
		private const double ERROR = 1e-12;

		private Triangle triangle; 

		[SetUp]
		public void Init () {
			triangle = (Triangle) LazyFactory.Create(typeof(Triangle));
		}

		[Test]
		public void TestArea() {
			triangle.area = 10;
			Assertion.AssertEquals("sideA*heightA/2", 10, triangle.Area(), ERROR);
		}
	
	
		[Test]
		public void TestHeightA()
		{
			triangle.sideA = 3;
			triangle.heightA = 4;

			Assertion.AssertEquals("sideA*heightA/2", 6, triangle.Area(), ERROR);
		}

		[Test]
		public void TestHeightB() {
			triangle.sideB = 3;
			triangle.heightB = 4;

			Assertion.AssertEquals("sideB*heightB/2", 6, triangle.Area(), ERROR);
		}

		[Test]
		public void TestHeightC() {
			triangle.sideC = 3;
			triangle.heightC = 4;

			Assertion.AssertEquals("sideC*heightC/2", 6, triangle.Area(), ERROR);
		}

		[Test]
		public void Test1SideA() {
			triangle.sideA = 6;
			triangle.AngleB = Math.PI/3;
			triangle.AngleC = Math.PI/3;

			Assertion.AssertEquals( Math.PI/3, triangle.AngleA, ERROR );
			Assertion.AssertEquals( Math.Sqrt(9*3*3*3), triangle.Area(), ERROR);
		}

		[Test]
		public void Test1SideB() {
			triangle.sideB = 6;
			triangle.AngleA = Math.PI/3;
			triangle.AngleC = Math.PI/3;

			Assertion.AssertEquals( Math.PI/3, triangle.AngleB, ERROR );
			Assertion.AssertEquals( Math.Sqrt(9*3*3*3), triangle.Area(), ERROR);
		}

		[Test]
		public void Test1SideC() {
			triangle.sideC= 6;
			triangle.AngleA = Math.PI/3;
			triangle.AngleB = Math.PI/3;

			Assertion.AssertEquals( Math.PI/3, triangle.AngleC, ERROR );
			Assertion.AssertEquals( Math.Sqrt(9*3*3*3), triangle.Area(), ERROR);
		}

		[Test]
		public void Test1Side3Angles () {
			triangle.sideA= 6;
			triangle.AngleA = Math.PI/3;
			triangle.AngleB = Math.PI/3;
			triangle.AngleC = Math.PI/3;

			Assertion.AssertEquals( Math.Sqrt(9*3*3*3), triangle.Area(), ERROR);
		}

		[Test]
		public void Test1Side2AnglesX() {
			triangle.sideA= 6;
			triangle.AngleA = Math.PI/3;
			triangle.AngleB = Math.PI/3;

			Assertion.AssertEquals( Math.PI/3, triangle.AngleC, ERROR );
			Assertion.AssertEquals( Math.Sqrt(9*3*3*3), triangle.Area(), ERROR);
		}

		[Test]
		public void Test2SidesA() {
			triangle.AngleA = Math.PI/2;
			triangle.sideB = 4;
			triangle.sideC = 3;	

			Assertion.AssertEquals("2 sides A", 6, triangle.Area(), ERROR);
		}

		[Test]
		public void Test2SidesB() {
			triangle.AngleB = Math.PI/2;
			triangle.sideA = 4;
			triangle.sideC = 3;	

			Assertion.AssertEquals("2 sides B", 6, triangle.Area(), ERROR);
		}

		[Test]
		public void Test2SidesC() {
			triangle.AngleC = Math.PI/2;
			triangle.sideA = 4;
			triangle.sideB = 3;	

			Assertion.AssertEquals("2 sides C", 6, triangle.Area(), ERROR);
		}

		[Test]
		public void Test3SidesRectangle() {
			triangle.sideA = 3;
			triangle.sideB = 4;
			triangle.sideC = 5;

			Assertion.AssertEquals(6, triangle.Area(), ERROR);
		}

		[Test]
		public void Test3SidesEquilateral() {
			triangle.sideA = 6;
			triangle.sideB = 6;
			triangle.sideC = 6;

			Assertion.AssertEquals( Math.Sqrt(9*3*3*3), triangle.Area(), ERROR);
		}
	}
}
