using System;
using System.Reflection;
using System.Reflection.Emit;

namespace Lazy
{
	/// <summary>
	/// Descripcin breve de LazyType.
	/// </summary>
	public class LazyType
	{
		public const String MODEL_FIELD = "__lazyModel";


		public static Type CreateType (LazyObjectModel model) {

			Type        lazyType;
			TypeBuilder typeBuilder = CreateTypeBuilder(model.Type);

			// Data definition: new reference to LazyObjectModel;

			FieldInfo modelField = typeBuilder.DefineField
				(MODEL_FIELD, typeof(LazyObjectModel), FieldAttributes.Public); // FieldAttributes.Private);

			// Constructor (MODEL_FIELD initialization)

			DefineConstructor(model.Type, typeBuilder, modelField);

			// Lazy method definition

			for (int i=0; i<model.LazyMethodCount; i++) {
				LazyMethod.Define(typeBuilder,model,i,modelField);
			}

			// Lazy type

			lazyType = typeBuilder.CreateType();

			//((AssemblyBuilder)lazyType.Assembly).Save(lazyType.FullName+".dll");

			return lazyType;
		}


		// TYPE BUILDER
		// ------------
        		
		
		private static TypeBuilder CreateTypeBuilder (Type baseType) {

			AppDomain domain = AppDomain.CurrentDomain;
			string newTypeName = baseType.Name+"Lazy";

			// Create a dynamic assembly.

			AssemblyName myAssemblyName = new AssemblyName();

			myAssemblyName.Name = newTypeName+"Assembly";

			AssemblyBuilder myAssembly =
				domain.DefineDynamicAssembly(myAssemblyName,AssemblyBuilderAccess.RunAndSave);
				//domain.DefineDynamicAssembly(myAssemblyName,AssemblyBuilderAccess.Save,".");
			
			// Define a dynamic module in the dynamic assembly.

			ModuleBuilder myModuleBuilder=myAssembly.DefineDynamicModule("LazyModule"); // ,newTypeName+".dll");		

			// Insert a public class in the assembly

			return myModuleBuilder.DefineType(newTypeName,TypeAttributes.Public,baseType);
		} 

				
		// CONSTRUCTOR
		// -----------
		//
		// public LazyClass()
		// {
		// 		model = LazyFactory.GetObjectModel( this.GetType().BaseType ); 
		// }
		//
		//.method public hidebysig specialname rtspecialname 
		//        instance void  .ctor() cil managed
		//{
		//  .maxstack  3
		//  IL_0000:  ldarg.0
		//  IL_0001:  call       instance void [mscorlib]System.Object::.ctor()
		//  IL_0006:  ldarg.0
		//  IL_0007:  ldarg.0
		//  IL_0008:  call       instance class [mscorlib]System.Type [mscorlib]System.Object::GetType()
		//  IL_000d:  callvirt   instance class [mscorlib]System.Type [mscorlib]System.Type::get_BaseType()
		//  IL_0012:  call       class [Lazy]Lazy.LazyObjectModel [Lazy]Lazy.LazyFactory::GetObjectModel(class [mscorlib]System.Type)
		//  IL_0017:  stfld      class [Lazy]Lazy.LazyObjectModel Lazy.Example.LazyClass::model
		//  IL_001c:  ret
		//} 

		private static void DefineConstructor (Type baseType, TypeBuilder myTypeBuilder, FieldInfo modelField) {
			
			Type[] constructorArgs = new Type[0];
			ConstructorBuilder constructor = myTypeBuilder.DefineConstructor(
				MethodAttributes.Public, CallingConventions.Standard, constructorArgs);

			ILGenerator constructorIL = constructor.GetILGenerator();

			// Llamada al constructor base

			constructorIL.Emit(OpCodes.Ldarg_0);   // this
			ConstructorInfo superConstructor = baseType.GetConstructor(constructorArgs);
			constructorIL.Emit(OpCodes.Call, superConstructor);

			// Inicializacin del modelo

			MethodInfo methodGetType = typeof(object).GetMethod("GetType");
			MethodInfo methodBaseType = typeof(Type).GetMethod("get_BaseType");
			MethodInfo methodGetModel = typeof(LazyFactory).GetMethod("GetObjectModel");

			constructorIL.Emit(OpCodes.Ldarg_0);   // this
			constructorIL.Emit(OpCodes.Ldarg_0);   // this
			constructorIL.Emit(OpCodes.Call, methodGetType);
			constructorIL.Emit(OpCodes.Callvirt, methodBaseType);
			constructorIL.Emit(OpCodes.Call, methodGetModel);

			constructorIL.Emit(OpCodes.Stfld, modelField);
			
			constructorIL.Emit(OpCodes.Ret);
		}
	}
}
