using System;
using System.Reflection;

using NUnit.Framework;

using Lazy;



namespace Lazy.Test
{
	/// <summary>
	/// Descripcin breve de Class1.
	/// </summary>
	/// 
	[TestFixture]
	public class LazyAttributesTest
	{
		[Test]
		public void TestLazy()
		{
			Type type = typeof(LazyClass);

			object[] attributes = type.GetCustomAttributes(typeof(LazyAttribute),false);

			Assertion.AssertEquals(1, attributes.Length);
			Assertion.AssertEquals(typeof(LazyAttribute), attributes[0].GetType());
		}

		[Test]
		public void TestAlternativeWithoutParams () 
		{
			Type       type = typeof(LazyClass);
			MethodInfo method = type.GetMethod("Method");
			MethodInfo alternative = type.GetMethod("MethodAlternative");

			CheckAlternativeImplementation (method, alternative);
		}

		[Test]
		public void TestAlternativeWithReturn () {
			Type       type = typeof(LazyClass);
			MethodInfo method = type.GetMethod("Method2");
			MethodInfo alternative = type.GetMethod("Method2Alternative");

			CheckAlternativeImplementation (method, alternative);
		}

		[Test]
		public void TestAlternativeWithParams () {
			Type       type = typeof(LazyClass);
			MethodInfo method = type.GetMethod("Method3");
			MethodInfo alternative = type.GetMethod("Method3Alternative");

			CheckAlternativeImplementation (method, alternative);
		}

		[Test]
		public void TestAlternativeWithOverload () {
			Type       type = typeof(LazyClass);
			MethodInfo method;
			MethodInfo alternative = type.GetMethod("Method4Alternative");

			// Parmetros

			ParameterInfo[] alternativeParameters = alternative.GetParameters();
			Type[]     parameters = new Type[alternativeParameters.Length];

			for (int i=0; i<parameters.Length; i++)
				parameters[i] = alternativeParameters[i].ParameterType;

			// Mtodo correspondiente

			method = type.GetMethod("Method4",parameters);

			CheckAlternativeImplementation (method, alternative);
		}


		private void CheckAlternativeImplementation (MethodInfo standard, MethodInfo alternative) 
		{
			object[]   attributes = alternative.GetCustomAttributes(typeof(AlternativeImplementationAttribute),false);

			Assertion.AssertEquals(1, attributes.Length);
			Assertion.AssertEquals(typeof(AlternativeImplementationAttribute), attributes[0].GetType());

			AlternativeImplementationAttribute attribute = (AlternativeImplementationAttribute) attributes[0];

			Assertion.AssertEquals(standard.Name, attribute.Method);

			Assertion.AssertEquals(standard.ReturnType,alternative.ReturnType);

			CheckParameters(standard,alternative);
		}

		private void CheckParameters (MethodInfo method1, MethodInfo method2) 
		{
			ParameterInfo[] alternativeParameters = method1.GetParameters();
			ParameterInfo[] actualParameters = method2.GetParameters();

			Console.Out.WriteLine(method1.Name + " <-> " + method2.Name );

			Assertion.AssertEquals(actualParameters.Length, alternativeParameters.Length);

			for (int i=0; i<actualParameters.Length; i++)  {
				Assertion.AssertEquals(actualParameters[i].ParameterType, alternativeParameters[i].ParameterType);	
				Console.Out.WriteLine(" "+actualParameters[i].ParameterType );
			}
		}
	}
}
