using System;
using System.Reflection;
using System.Reflection.Emit;

using NUnit.Framework;

using Reflector.Disassembler;


namespace ILReader.Test {
	/// <summary>
	/// Descripcin breve de Class1.
	/// </summary>
	/// 
	[TestFixture]
	public class ILReaderTest {

		private const BindingFlags methodFlags = 
			BindingFlags.Public 
			| BindingFlags.NonPublic 
			// | BindingFlags.Static 
			| BindingFlags.Instance 
			| BindingFlags.DeclaredOnly;


		Type         type;
		ModuleReader reader;

		[SetUp]
		public void SetUp () 
		{
			type   = this.GetType();
			reader = new ModuleReader(type.Module, new AssemblyProvider());
		}

		[Test]
		public void UsedTest() {

			Console.WriteLine();
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("USED (opcode ldfld)");
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("------------------------------------------------------------------------");

			foreach (MethodBase methodBase in type.GetMethods(methodFlags)) {
				MethodBody methodBody = reader.GetMethodBody(methodBase);		

				Console.WriteLine();
				Console.WriteLine(methodBase.ToString());
				Console.WriteLine("------------------------------------------------------------------------");
				UsedData(methodBody);
			}
		}


		[Test]
		public void DefinedTest() {

			Console.WriteLine();
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("DEFINED (opcode stfld)");
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("------------------------------------------------------------------------");

			foreach (MethodBase methodBase in type.GetMethods(methodFlags)) {
				MethodBody methodBody = reader.GetMethodBody(methodBase);
				
				Console.WriteLine();
				Console.WriteLine(methodBase.ToString());
				Console.WriteLine("------------------------------------------------------------------------");
				DefinedData(methodBody);
			}
		}

		[Test]
		public void CallTest() {

			Console.WriteLine();
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("CALLS (opcodes: call, callvirt, newobj)");
			Console.WriteLine("------------------------------------------------------------------------");
			Console.WriteLine("------------------------------------------------------------------------");

			foreach (MethodBase methodBase in type.GetMethods(methodFlags)) {
				MethodBody methodBody = reader.GetMethodBody(methodBase);
				
				Console.WriteLine();
				Console.WriteLine(methodBase.ToString());
				Console.WriteLine("------------------------------------------------------------------------");
				CalledMethods(methodBody);
			}
		}


		// Used fields (ldfld)

		private void UsedData (MethodBody methodBody) {

			foreach (Instruction instruction in methodBody.GetInstructions()) {

				OpCode code = instruction.Code;
				object operand = instruction.Operand;
				byte[] operandData = instruction.GetOperandData();
				
				if ( (operand!=null) 
					&& (code.OperandType == OperandType.InlineField)
					&& (code.Name.StartsWith("ld")) ) {
					
					Console.Write(code.Name);
					Console.Write(" ");

					FieldInfo memberInfo = (FieldInfo) operand;	
					
					Console.Write(memberInfo.DeclaringType);
					Console.Write("::");
					Console.WriteLine(memberInfo);
				}      
			}
		}

		// Defined fields (stfld)

		private void DefinedData (MethodBody methodBody) {

			foreach (Instruction instruction in methodBody.GetInstructions()) {

				OpCode code = instruction.Code;
				object operand = instruction.Operand;
				byte[] operandData = instruction.GetOperandData();
				
				if ( (operand!=null) 
					&& (code.OperandType == OperandType.InlineField)
					&& (code.Name.StartsWith("st")) ) {
					
					Console.Write(code.Name);
					Console.Write(" ");

					FieldInfo memberInfo = (FieldInfo) operand;
				
					Console.Write(memberInfo.DeclaringType);
					Console.Write("::");
					Console.WriteLine(memberInfo);
				}      
			}
		}

		// Called methods (call|callvirt|newobj)

		private void CalledMethods (MethodBody methodBody) {

			foreach (Instruction instruction in methodBody.GetInstructions()) {

				OpCode code = instruction.Code;
				object operand = instruction.Operand;
				byte[] operandData = instruction.GetOperandData();

				if (  (operand != null)
				   && (code.OperandType == OperandType.InlineMethod) ) {
				
					Console.Write(code.Name);
					Console.Write(" ");

					// MethodBase = MethodInfo || ConstructorInfo
					MethodBase memberInfo = (MethodBase) operand;
					
					Console.Write(memberInfo.DeclaringType);
					Console.Write("::");
					Console.WriteLine(memberInfo);
				}      
			}
		}


		private sealed class AssemblyProvider : IAssemblyProvider {
			public Assembly Load(string assemblyName) {
				return Assembly.Load(assemblyName);	
			}
			
			public Assembly[] GetAssemblies() {
				throw new NotImplementedException();	
			}
		}
	}
}

